@@ -370,7 +370,7 @@ def create_rings_from_ways(way_ids, ways, nodes):
     
     return rings
 
-def load_gdf_from_openstreetmap(rectangle_vertices):
+def load_gdf_from_openstreetmap(rectangle_vertices, floor_height=3.0):
     """Download and process building footprint data from OpenStreetMap.
     
     This function:
@@ -398,7 +398,12 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
     max_lat = max(v[1] for v in rectangle_vertices)
     
     # Enhanced Overpass API query with recursive member extraction
-    overpass_url = "http://overpass-api.de/api/interpreter"
+    # Try multiple Overpass endpoints to improve resiliency against rate limits or outages
+    overpass_endpoints = [
+        "https://overpass-api.de/api/interpreter",
+        "https://overpass.kumi.systems/api/interpreter",
+        "https://overpass.openstreetmap.ru/api/interpreter",
+    ]
     overpass_query = f"""
     [out:json];
     (
@@ -412,9 +417,40 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
     out geom;
     """
     
-    # Send the request to the Overpass API
-    response = requests.get(overpass_url, params={'data': overpass_query})
-    data = response.json()
+    # Send the request to the Overpass API with fallbacks and robust JSON handling
+    headers = {"User-Agent": "voxcity/ci (https://github.com/voxcity)"}
+    data = None
+    last_error = None
+    for overpass_url in overpass_endpoints:
+        try:
+            response = requests.get(overpass_url, params={"data": overpass_query}, headers=headers, timeout=30)
+            # Ensure HTTP OK
+            if response.status_code != 200:
+                last_error = Exception(f"HTTP {response.status_code} from {overpass_url}")
+                continue
+            # Some servers return HTML/plain text on rate-limit; guard JSON parsing
+            content_type = response.headers.get("Content-Type", "")
+            if "json" not in content_type.lower():
+                # Attempt JSON anyway; if fails, try next endpoint
+                try:
+                    data = response.json()
+                except Exception as e:
+                    last_error = e
+                    continue
+            else:
+                data = response.json()
+            # Validate structure
+            if not isinstance(data, dict) or "elements" not in data:
+                last_error = Exception(f"Malformed Overpass response from {overpass_url}")
+                data = None
+                continue
+            # Success
+            break
+        except Exception as e:
+            last_error = e
+            continue
+    if data is None:
+        raise RuntimeError(f"Failed to fetch OSM data from Overpass endpoints. Last error: {last_error}")
     
     # Build a mapping from (type, id) to element
     id_map = {}
@@ -435,7 +471,7 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
         """
         return [coord for coord in geometry]  # Keep original order since already (lon, lat)
     
-    def get_height_from_properties(properties):
+    def get_height_from_properties(properties, floor_height=3.0):
         """Helper function to extract height from properties.
         
         Args:
@@ -451,9 +487,25 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
             except ValueError:
                 pass
         
+        # Infer from floors when available
+        floors_candidates = [
+            properties.get('building:levels'),
+            properties.get('levels'),
+            properties.get('num_floors')
+        ]
+        for floors in floors_candidates:
+            if floors is None:
+                continue
+            try:
+                floors_val = float(floors)
+                if floors_val > 0:
+                    return float(floor_height) * floors_val
+            except ValueError:
+                continue
+        
         return 0  # Default height if no valid height found
     
-    def extract_properties(element):
+    def extract_properties(element, floor_height=3.0):
         """Helper function to extract and process properties from an element.
         
         Args:
@@ -465,7 +517,7 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
         properties = element.get('tags', {})
         
         # Get height (now using the helper function)
-        height = get_height_from_properties(properties)
+        height = get_height_from_properties(properties, floor_height=floor_height)
             
         # Get min_height and min_level
         min_height = properties.get('min_height', '0')
@@ -490,7 +542,7 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
             "is_inner": False,
             "levels": levels,
             "height_source": "explicit" if properties.get('height') or properties.get('building:height') 
-                               else "levels" if levels is not None 
+                               else "levels" if (levels is not None) or (properties.get('num_floors') is not None)
                                else "default",
             "min_level": min_level if min_level != '0' else None,
             "building": properties.get('building', 'no'),
@@ -548,13 +600,13 @@ def load_gdf_from_openstreetmap(rectangle_vertices):
         if element['type'] == 'way':
             if 'geometry' in element:
                 coords = [(node['lon'], node['lat']) for node in element['geometry']]
-                properties = extract_properties(element)
+                properties = extract_properties(element, floor_height=floor_height)
                 feature = create_polygon_feature(coords, properties)
                 if feature:
                     features.append(feature)
                     
         elif element['type'] == 'relation':
-            properties = extract_properties(element)
+            properties = extract_properties(element, floor_height=floor_height)
             
             # Process each member of the relation
             for member in element['members']:
@@ -932,14 +984,43 @@ def load_land_cover_gdf_from_osm(rectangle_vertices_ori):
         "out skel qt;"
     )
 
-    # Overpass API endpoint
-    overpass_url = "http://overpass-api.de/api/interpreter"
+    # Overpass API endpoints (fallbacks)
+    overpass_endpoints = [
+        "https://overpass-api.de/api/interpreter",
+        "https://overpass.kumi.systems/api/interpreter",
+        "https://overpass.openstreetmap.ru/api/interpreter",
+    ]
 
-    # Fetch data from Overpass API
+    # Fetch data from Overpass API with fallbacks and robust JSON handling
     print("Fetching data from Overpass API...")
-    response = requests.get(overpass_url, params={'data': query})
-    response.raise_for_status()
-    data = response.json()
+    headers = {"User-Agent": "voxcity/ci (https://github.com/voxcity)"}
+    data = None
+    last_error = None
+    for overpass_url in overpass_endpoints:
+        try:
+            response = requests.get(overpass_url, params={'data': query}, headers=headers, timeout=30)
+            if response.status_code != 200:
+                last_error = Exception(f"HTTP {response.status_code} from {overpass_url}")
+                continue
+            content_type = response.headers.get("Content-Type", "")
+            if "json" not in content_type.lower():
+                try:
+                    data = response.json()
+                except Exception as e:
+                    last_error = e
+                    continue
+            else:
+                data = response.json()
+            if not isinstance(data, dict) or 'elements' not in data:
+                last_error = Exception(f"Malformed Overpass response from {overpass_url}")
+                data = None
+                continue
+            break
+        except Exception as e:
+            last_error = e
+            continue
+    if data is None:
+        raise RuntimeError(f"Failed to fetch OSM data from Overpass endpoints. Last error: {last_error}")
 
     # Convert OSM data to GeoJSON format using our custom converter instead of json2geojson
     print("Converting data to GeoJSON format...")
