@@ -669,6 +669,13 @@ def export_netcdf_to_obj(
     stride_scalar=(1, 1, 1),
     contour_levels=24,
     cmap_name="magma",
+    vmin=None,
+    vmax=None,
+    iso_vmin=None,
+    iso_vmax=None,
+    greedy_vox=True,
+    vox_voxel_size=None,
+    scalar_spacing=None,
     opacity_points=None,
     max_opacity=0.10,
     classes_to_show=None,
@@ -697,6 +704,15 @@ def export_netcdf_to_obj(
         stride_scalar (tuple[int,int,int]): Downsampling strides for scalar (k,j,i).
         contour_levels (int): Number of iso-surface levels between vmin and vmax.
         cmap_name (str): Matplotlib colormap name for iso-surfaces.
+        vmin (float|None): Minimum scalar value for color mapping and iso range. If None, inferred.
+        vmax (float|None): Maximum scalar value for color mapping and iso range. If None, inferred.
+        iso_vmin (float|None): Minimum scalar value to generate iso-surface levels. If None, uses vmin.
+        iso_vmax (float|None): Maximum scalar value to generate iso-surface levels. If None, uses vmax.
+        greedy_vox (bool): If True, use greedy meshing for VoxCity faces to reduce triangles.
+        vox_voxel_size (float|tuple[float,float,float]|None): If provided, overrides VoxCity voxel spacing
+            for X,Y,Z respectively in meters. A single float applies to all axes.
+        scalar_spacing (tuple[float,float,float]|None): If provided, overrides scalar grid spacing (dx,dy,dz)
+            used for iso-surface generation. Values are in meters.
         opacity_points (list[tuple[float,float]]|None): Transfer function control points (value, alpha in [0..1]).
         max_opacity (float): Global max opacity multiplier for iso-surfaces (0..1).
         classes_to_show (set[int]|None): Optional subset of voxel classes to export; None -> all present (except 0).
@@ -900,12 +916,173 @@ def export_netcdf_to_obj(
         mesh.visual.face_colors = np.tile(rgba, (len(F), 1))
         return mesh, len(F)
 
-    def build_tm_isosurfaces_regular_grid(A_scalar, vmin, vmax, levels, dx, dy, dz, origin_xyz, cmap_name, opacity_points, max_opacity):
+    def _greedy_rectangles(mask2d):
+        h, w = mask2d.shape
+        visited = np.zeros_like(mask2d, dtype=bool)
+        rects = []
+        for u in range(h):
+            v = 0
+            while v < w:
+                if visited[u, v] or not mask2d[u, v]:
+                    v += 1
+                    continue
+                # width
+                width = 1
+                while v + width < w and mask2d[u, v + width] and not visited[u, v + width]:
+                    width += 1
+                # height
+                height = 1
+                done = False
+                while u + height < h and not done:
+                    for k_ in range(width):
+                        if (not mask2d[u + height, v + k_]) or visited[u + height, v + k_]:
+                            done = True
+                            break
+                    if not done:
+                        height += 1
+                visited[u:u + height, v:v + width] = True
+                rects.append((u, v, height, width))
+                v += width
+        return rects
+
+    def make_voxel_mesh_uniform_color_greedy(occ_mask, X, Y, Z, rgb, name="class"):
+        posx, negx, posy, negy, posz, negz = _exposed_face_masks(occ_mask.astype(bool))
+        total_faces_naive = int(posx.sum() + negx.sum() + posy.sum() + negy.sum() + posz.sum() + negz.sum())
+        if total_faces_naive == 0:
+            return None, 0
+
+        dx = (X[1] - X[0]) if len(X) > 1 else 1.0
+        dy = (Y[1] - Y[0]) if len(Y) > 1 else 1.0
+        dz = (Z[1] - Z[0]) if len(Z) > 1 else 1.0
+        hx, hy, hz = dx / 2.0, dy / 2.0, dz / 2.0
+
+        V_list = []
+        F_list = []
+        start_idx = 0
+
+        def add_quad(P, Q, R, S, order):
+            nonlocal start_idx
+            V_list.extend([P, Q, R, S])
+            a = start_idx
+            b = start_idx + 1
+            c = start_idx + 2
+            d = start_idx + 3
+            start_idx += 4
+            if order == "default":
+                F_list.append([a, b, c])
+                F_list.append([a, c, d])
+            else:  # flip
+                F_list.append([a, c, b])
+                F_list.append([a, d, c])
+
+        K, J, I = occ_mask.shape
+
+        # +x and -x: iterate i, mask over (k,j)
+        for plane, mask3 in (("+x", posx), ("-x", negx)):
+            order = "default"
+            for i in range(I):
+                m2 = mask3[:, :, i]
+                if not np.any(m2):
+                    continue
+                for u, v, h, w in _greedy_rectangles(m2):
+                    k0, j0 = u, v
+                    k1, j1 = u + h, v + w
+                    z0 = Z[k0] - hz
+                    z1 = Z[k1 - 1] + hz
+                    y0 = Y[j0] - hy
+                    y1 = Y[j1 - 1] + hy
+                    x_center = X[i]
+                    if plane == "+x":
+                        x = x_center + hx
+                        P = (x, y0, z0)
+                        Q = (x, y1, z0)
+                        R = (x, y1, z1)
+                        S = (x, y0, z1)
+                    else:  # -x
+                        x = x_center - hx
+                        P = (x, y0, z1)
+                        Q = (x, y1, z1)
+                        R = (x, y1, z0)
+                        S = (x, y0, z0)
+                    add_quad(P, Q, R, S, order)
+
+        # +y and -y: iterate j, mask over (k,i)
+        for plane, mask3 in (("+y", posy), ("-y", negy)):
+            order = "flip"  # enforce outward normals like original
+            for j in range(J):
+                m2 = mask3[:, j, :]
+                if not np.any(m2):
+                    continue
+                for u, v, h, w in _greedy_rectangles(m2):
+                    k0, i0 = u, v
+                    k1, i1 = u + h, v + w
+                    z0 = Z[k0] - hz
+                    z1 = Z[k1 - 1] + hz
+                    x0 = X[i0] - hx
+                    x1 = X[i1 - 1] + hx
+                    y_center = Y[j]
+                    if plane == "+y":
+                        y = y_center + hy
+                        P = (x0, y, z0)
+                        Q = (x1, y, z0)
+                        R = (x1, y, z1)
+                        S = (x0, y, z1)
+                    else:  # -y
+                        y = y_center - hy
+                        P = (x0, y, z1)
+                        Q = (x1, y, z1)
+                        R = (x1, y, z0)
+                        S = (x0, y, z0)
+                    add_quad(P, Q, R, S, order)
+
+        # +z and -z: iterate k, mask over (j,i)
+        for plane, mask3 in (("+z", posz), ("-z", negz)):
+            order = "default"
+            for k in range(K):
+                m2 = mask3[k, :, :]
+                if not np.any(m2):
+                    continue
+                for u, v, h, w in _greedy_rectangles(m2):
+                    j0, i0 = u, v
+                    j1, i1 = u + h, v + w
+                    y0 = Y[j0] - hy
+                    y1 = Y[j1 - 1] + hy
+                    x0 = X[i0] - hx
+                    x1 = X[i1 - 1] + hx
+                    z_center = Z[k]
+                    if plane == "+z":
+                        z = z_center + hz
+                        P = (x0, y0, z)
+                        Q = (x1, y0, z)
+                        R = (x1, y1, z)
+                        S = (x0, y1, z)
+                    else:  # -z
+                        z = z_center - hz
+                        P = (x0, y1, z)
+                        Q = (x1, y1, z)
+                        R = (x1, y0, z)
+                        S = (x0, y0, z)
+                    add_quad(P, Q, R, S, order)
+
+        if not V_list or not F_list:
+            return None, 0
+        V = np.asarray(V_list, dtype=np.float64)
+        F = np.asarray(F_list, dtype=np.int64)
+        mesh = trimesh.Trimesh(vertices=V, faces=F, process=False)
+        rgba = np.array([int(rgb[0]), int(rgb[1]), int(rgb[2]), 255], dtype=np.uint8)
+        mesh.visual.face_colors = np.tile(rgba, (len(F), 1))
+        return mesh, len(F)
+
+    def build_tm_isosurfaces_regular_grid(A_scalar, vmin, vmax, levels, dx, dy, dz, origin_xyz, cmap_name, opacity_points, max_opacity, iso_vmin=None, iso_vmax=None):
         cmap = cm.get_cmap(cmap_name)
         meshes = []
         if levels <= 0:
             return meshes
-        iso_vals = np.linspace(vmin, vmax, int(levels))
+        ivmin = vmin if (iso_vmin is None) else float(iso_vmin)
+        ivmax = vmax if (iso_vmax is None) else float(iso_vmax)
+        if not (ivmin < ivmax):
+            return meshes
+        iso_vals = np.linspace(ivmin, ivmax, int(levels))
         for iso in iso_vals:
             a_base = float(opacity_at(iso, opacity_points or []))
             a_base = min(max(a_base, 0.0), 1.0)
@@ -1053,9 +1230,23 @@ def export_netcdf_to_obj(
     # Y flip (north-up)
     Av_kji = Av_kji[:, ::-1, :]
 
-    Zv_s = Zv[:: max(1, svz)].astype(float)
-    Yv_s = (Yv[:: max(1, svy)] - Yv.min()).astype(float)
-    Xv_s = (Xv[:: max(1, svx)] - Xv.min()).astype(float)
+    # VoxCity coordinate spacing (optionally override by vox_voxel_size)
+    Ks, Js, Is = Av_kji.shape
+    if vox_voxel_size is None:
+        Zv_s = Zv[:: max(1, svz)].astype(float)
+        Yv_s = (Yv[:: max(1, svy)] - Yv.min()).astype(float)
+        Xv_s = (Xv[:: max(1, svx)] - Xv.min()).astype(float)
+    else:
+        if isinstance(vox_voxel_size, (int, float)):
+            vx = vy = vz = float(vox_voxel_size)
+        else:
+            try:
+                vx, vy, vz = (float(vox_voxel_size[0]), float(vox_voxel_size[1]), float(vox_voxel_size[2]))
+            except Exception as e:
+                raise ValueError("vox_voxel_size must be a float or a length-3 iterable of floats (vx,vy,vz)") from e
+        Xv_s = (np.arange(Is, dtype=float) * vx)
+        Yv_s = (np.arange(Js, dtype=float) * vy)
+        Zv_s = (np.arange(Ks, dtype=float) * vz)
 
     # Load scalar and georeference using lon/lat table
     dss = xr.open_dataset(scalar_nc, decode_coords="all", decode_times=True)
@@ -1123,14 +1314,32 @@ def export_netcdf_to_obj(
     finite_vals = A_s[np.isfinite(A_s)]
     if finite_vals.size == 0:
         raise RuntimeError("No finite scalar values after masking.")
-    vmin, vmax = clip_minmax(finite_vals, 0.0)
+    if (vmin is None) or (vmax is None):
+        auto_vmin, auto_vmax = clip_minmax(finite_vals, 0.0)
+        if vmin is None:
+            vmin = auto_vmin
+        if vmax is None:
+            vmax = auto_vmax
+    if not (vmin < vmax):
+        raise ValueError("vmin must be less than vmax.")
     A_s[np.isnan(A_s)] = vmin - 1e6
 
+    # Determine iso-surface generation range (defaults to color mapping range)
+    iso_vmin_eff = vmin if (iso_vmin is None) else float(iso_vmin)
+    iso_vmax_eff = vmax if (iso_vmax is None) else float(iso_vmax)
+    if not (iso_vmin_eff < iso_vmax_eff):
+        raise ValueError("iso_vmin must be less than iso_vmax.")
+
     Xmin, Xmax = np.nanmin(Xs_m), np.nanmax(Xs_m)
     Ymin, Ymax = np.nanmin(Ys_m), np.nanmax(Ys_m)
     dx_s = (Xmax - Xmin) / max(1, Is - 1)
     dy_s = (Ymax - Ymin) / max(1, Js - 1)
     dz_s = (Zk[-1] - Zk[0]) / max(1, Ks - 1) if Ks > 1 else 1.0
+    if scalar_spacing is not None:
+        try:
+            dx_s, dy_s, dz_s = (float(scalar_spacing[0]), float(scalar_spacing[1]), float(scalar_spacing[2]))
+        except Exception as e:
+            raise ValueError("scalar_spacing must be a length-3 iterable of floats (dx,dy,dz)") from e
     origin_xyz = (float(Xmin), float(Ymin), float(Zk[0]))
 
     vox_meshes = {}
@@ -1149,7 +1358,10 @@ def export_netcdf_to_obj(
         if not np.any(mask):
             continue
         rgb = voxel_color_map.get(int(cls), [200, 200, 200])
-        m_cls, faces = make_voxel_mesh_uniform_color(mask, Xv_s, Yv_s, Zv_s, rgb=rgb, name=f"class_{int(cls)}")
+        if greedy_vox:
+            m_cls, faces = make_voxel_mesh_uniform_color_greedy(mask, Xv_s, Yv_s, Zv_s, rgb=rgb, name=f"class_{int(cls)}")
+        else:
+            m_cls, faces = make_voxel_mesh_uniform_color(mask, Xv_s, Yv_s, Zv_s, rgb=rgb, name=f"class_{int(cls)}")
         if m_cls is not None:
             vox_meshes[f"voxclass_{int(cls)}"] = m_cls
             faces_total += faces
@@ -1167,6 +1379,8 @@ def export_netcdf_to_obj(
         cmap_name=cmap_name,
         opacity_points=opacity_points,
         max_opacity=max_opacity,
+        iso_vmin=iso_vmin_eff,
+        iso_vmax=iso_vmax_eff,
     )
     for iso, m, rgba in iso_meshes:
         tm_meshes[f"iso_{iso:.6f}"] = m
