@@ -263,14 +263,19 @@ def calculate_grid_size(side_1, side_2, u_vec, v_vec, meshsize):
         >>> mesh = 10                    # Desired 10-unit mesh
         >>> grid_size, adj_mesh = calculate_grid_size(side1, side2, u, v, mesh)
     """
-    # Calculate number of cells needed in each direction, rounding to nearest integer
-    grid_size_0 = int(np.linalg.norm(side_1) / np.linalg.norm(meshsize * u_vec) + 0.5)
-    grid_size_1 = int(np.linalg.norm(side_2) / np.linalg.norm(meshsize * v_vec) + 0.5)
-    
-    # Adjust mesh sizes to exactly fit the desired area with the calculated number of cells
-    adjusted_mesh_size_0 = meshsize *  np.linalg.norm(meshsize * u_vec) * grid_size_0 / np.linalg.norm(side_1)
-    adjusted_mesh_size_1 = meshsize *  np.linalg.norm(meshsize * v_vec) * grid_size_1 / np.linalg.norm(side_2)
-    
+    # Calculate total side lengths in meters using the relationship between side vectors and unit vectors
+    # u_vec and v_vec represent degrees per meter along each side direction
+    dist_side_1_m = np.linalg.norm(side_1) / (np.linalg.norm(u_vec) + 1e-12)
+    dist_side_2_m = np.linalg.norm(side_2) / (np.linalg.norm(v_vec) + 1e-12)
+
+    # Calculate number of cells (nx along u, ny along v), rounding to nearest integer and ensuring at least 1
+    grid_size_0 = max(1, int(dist_side_1_m / meshsize + 0.5))
+    grid_size_1 = max(1, int(dist_side_2_m / meshsize + 0.5))
+
+    # Adjust mesh sizes (in meters) to exactly fit the sides with the calculated number of cells
+    adjusted_mesh_size_0 = dist_side_1_m / grid_size_0
+    adjusted_mesh_size_1 = dist_side_2_m / grid_size_1
+
     return (grid_size_0, grid_size_1), (adjusted_mesh_size_0, adjusted_mesh_size_1)
 
 def create_coordinate_mesh(origin, grid_size, adjusted_meshsize, u_vec, v_vec):
@@ -908,12 +913,19 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
     Process buildings using fast rasterio-based approach.
     Faster but less precise for overlapping footprints.
     """
-    # Set up transform for rasterio
-    min_x, min_y = origin[0], origin[1]
-    max_corner = origin + grid_size[0] * adjusted_meshsize[0] * u_vec + grid_size[1] * adjusted_meshsize[1] * v_vec
-    max_x, max_y = max_corner[0], max_corner[1]
-    
-    transform = from_bounds(min_x, min_y, max_x, max_y, grid_size[0], grid_size[1])
+    # Set up transform for rasterio using rotated basis defined by u_vec and v_vec
+    # Step vectors in coordinate units (degrees) per cell
+    u_step = adjusted_meshsize[0] * u_vec
+    v_step = adjusted_meshsize[1] * v_vec
+
+    # Define the top-left corner so that row=0 is the northern edge
+    top_left = origin + grid_size[1] * v_step
+
+    # Affine transform mapping (col, row) -> (x, y)
+    # x = a*col + b*row + c ; y = d*col + e*row + f
+    # col increases along u_step; row increases southward, hence -v_step
+    transform = Affine(u_step[0], -v_step[0], top_left[0],
+                       u_step[1], -v_step[1], top_left[1])
     
     # Process buildings data
     filtered_gdf = filtered_gdf.copy()
@@ -926,6 +938,12 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
     
     if 'is_inner' not in filtered_gdf.columns:
         filtered_gdf['is_inner'] = False
+    else:
+        # Ensure boolean dtype with NaN treated as False for safe boolean operations
+        try:
+            filtered_gdf['is_inner'] = filtered_gdf['is_inner'].fillna(False).astype(bool)
+        except Exception:
+            filtered_gdf['is_inner'] = False
     
     if 'id' not in filtered_gdf.columns:
         filtered_gdf['id'] = range(len(filtered_gdf))
@@ -934,9 +952,9 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
     regular_buildings = filtered_gdf[~filtered_gdf['is_inner']].copy()
     regular_buildings = regular_buildings.sort_values('height', ascending=True, na_position='first')
     
-    # Initialize grids
-    building_height_grid = np.zeros(grid_size, dtype=np.float64)
-    building_id_grid = np.zeros(grid_size, dtype=np.float64)
+    # Temporary raster grids in rasterio's (rows=ny, cols=nx) order
+    height_raster = np.zeros((grid_size[1], grid_size[0]), dtype=np.float64)
+    id_raster = np.zeros((grid_size[1], grid_size[0]), dtype=np.float64)
     
     # Vectorized rasterization
     if len(regular_buildings) > 0:
@@ -949,9 +967,9 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
                            if pd.notna(height) and height > 0]
             
             if height_shapes:
-                building_height_grid = features.rasterize(
+                height_raster = features.rasterize(
                     height_shapes,
-                    out_shape=grid_size,
+                    out_shape=(grid_size[1], grid_size[0]),
                     transform=transform,
                     fill=0,
                     dtype=np.float64
@@ -962,9 +980,9 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
                         zip(valid_buildings.geometry, valid_buildings['id'])]
             
             if id_shapes:
-                building_id_grid = features.rasterize(
+                id_raster = features.rasterize(
                     id_shapes,
-                    out_shape=grid_size,
+                    out_shape=(grid_size[1], grid_size[0]),
                     transform=transform,
                     fill=0,
                     dtype=np.float64
@@ -977,17 +995,17 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
         if inner_shapes:
             inner_mask = features.rasterize(
                 inner_shapes,
-                out_shape=grid_size,
+                out_shape=(grid_size[1], grid_size[0]),
                 transform=transform,
                 fill=0,
                 dtype=np.uint8
             )
-            building_height_grid[inner_mask > 0] = 0
-            building_id_grid[inner_mask > 0] = 0
+            height_raster[inner_mask > 0] = 0
+            id_raster[inner_mask > 0] = 0
     
     # Simplified min_height grid 
     building_min_height_grid = np.empty(grid_size, dtype=object)
-    min_heights = np.zeros(grid_size, dtype=np.float64)
+    min_heights_raster = np.zeros((grid_size[1], grid_size[0]), dtype=np.float64)
     
     if len(regular_buildings) > 0:
         valid_buildings = regular_buildings[regular_buildings.geometry.is_valid].copy()
@@ -997,27 +1015,27 @@ def _process_with_rasterio(filtered_gdf, grid_size, adjusted_meshsize, origin, u
                                if pd.notna(min_h)]
             
             if min_height_shapes:
-                min_heights = features.rasterize(
+                min_heights_raster = features.rasterize(
                     min_height_shapes,
-                    out_shape=grid_size,
+                    out_shape=(grid_size[1], grid_size[0]),
                     transform=transform,
                     fill=0,
                     dtype=np.float64
                 )
     
     # Convert to list format (simplified)
+    # Convert raster (ny, nx) to internal orientation (nx, ny) with north-up
+    building_height_grid = np.flipud(height_raster).T
+    building_id_grid = np.flipud(id_raster).T
+    min_heights = np.flipud(min_heights_raster).T
+
     for i in range(grid_size[0]):
         for j in range(grid_size[1]):
             if building_height_grid[i, j] > 0:
                 building_min_height_grid[i, j] = [[min_heights[i, j], building_height_grid[i, j]]]
             else:
                 building_min_height_grid[i, j] = []
-    
-    # Fix north-south orientation by flipping grids
-    building_height_grid = np.flipud(building_height_grid)
-    building_id_grid = np.flipud(building_id_grid)
-    building_min_height_grid = np.flipud(building_min_height_grid)
-    
+
     return building_height_grid, building_min_height_grid, building_id_grid, filtered_gdf
 
 def create_building_height_grid_from_open_building_temporal_polygon(meshsize, rectangle_vertices, output_dir):        
@@ -1553,4 +1571,169 @@ def create_dem_grid_from_gdf_polygon(terrain_gdf, mesh_size, polygon):
     # By default, row=0 is the "north/top" row, row=-1 is "south/bottom" row.
     # If you prefer the bottom row as index=0, you'd do: np.flipud(dem_grid)
 
-    return np.flipud(dem_grid)
\ No newline at end of file
+    return np.flipud(dem_grid)
+
+def create_canopy_grids_from_tree_gdf(tree_gdf, meshsize, rectangle_vertices):
+    """
+    Create canopy top and bottom height grids from a tree GeoDataFrame.
+
+    Assumptions:
+    - Each tree is a point with attributes: 'top_height', 'bottom_height', 'crown_diameter'.
+    - The crown is modeled as a solid of revolution with an ellipsoidal vertical profile.
+      For a tree with top H_t, bottom H_b and crown radius R = crown_diameter/2,
+      at a horizontal distance r (r <= R) from the tree center:
+        z_top(r) = z0 + a * sqrt(1 - (r/R)^2)
+        z_bot(r) = z0 - a * sqrt(1 - (r/R)^2)
+      where a = (H_t - H_b)/2 and z0 = (H_t + H_b)/2.
+
+    The function outputs two grids (shape: (nx, ny) consistent with other grid functions):
+    - canopy_height_grid: maximum canopy top height per cell across trees
+    - canopy_bottom_height_grid: maximum canopy bottom height per cell across trees
+
+    Args:
+        tree_gdf (geopandas.GeoDataFrame): Tree points with required columns.
+        meshsize (float): Grid spacing in meters.
+        rectangle_vertices (list[tuple]): 4 vertices [(lon,lat), ...] defining the grid rectangle.
+
+    Returns:
+        tuple[np.ndarray, np.ndarray]: (canopy_height_grid, canopy_bottom_height_grid)
+    """
+
+    # Validate and prepare input GeoDataFrame
+    if tree_gdf is None or len(tree_gdf) == 0:
+        return np.array([]), np.array([])
+
+    required_cols = ['top_height', 'bottom_height', 'crown_diameter', 'geometry']
+    for col in required_cols:
+        if col not in tree_gdf.columns:
+            raise ValueError(f"tree_gdf must contain '{col}' column.")
+
+    # Ensure CRS is WGS84
+    if tree_gdf.crs is None:
+        warnings.warn("tree_gdf has no CRS. Assuming EPSG:4326.")
+        tree_gdf = tree_gdf.set_crs(epsg=4326)
+    elif tree_gdf.crs.to_epsg() != 4326:
+        tree_gdf = tree_gdf.to_crs(epsg=4326)
+
+    # Grid setup consistent with building/land cover grid functions
+    geod = initialize_geod()
+    vertex_0, vertex_1, vertex_3 = rectangle_vertices[0], rectangle_vertices[1], rectangle_vertices[3]
+
+    dist_side_1 = calculate_distance(geod, vertex_0[0], vertex_0[1], vertex_1[0], vertex_1[1])
+    dist_side_2 = calculate_distance(geod, vertex_0[0], vertex_0[1], vertex_3[0], vertex_3[1])
+
+    side_1 = np.array(vertex_1) - np.array(vertex_0)
+    side_2 = np.array(vertex_3) - np.array(vertex_0)
+    u_vec = normalize_to_one_meter(side_1, dist_side_1)
+    v_vec = normalize_to_one_meter(side_2, dist_side_2)
+
+    origin = np.array(rectangle_vertices[0])
+    grid_size, adjusted_meshsize = calculate_grid_size(side_1, side_2, u_vec, v_vec, meshsize)
+
+    nx, ny = grid_size[0], grid_size[1]
+
+    # Precompute metric cell-center coordinates in grid's (u,v) metric space (meters from origin)
+    i_centers_m = (np.arange(nx) + 0.5) * adjusted_meshsize[0]
+    j_centers_m = (np.arange(ny) + 0.5) * adjusted_meshsize[1]
+
+    # Initialize output grids
+    canopy_top = np.zeros((nx, ny), dtype=float)
+    canopy_bottom = np.zeros((nx, ny), dtype=float)
+
+    # Matrix to convert lon/lat offsets to metric (u,v) using u_vec, v_vec
+    # delta_lonlat ≈ [u_vec v_vec] @ [alpha; beta], where alpha/beta are meters along u/v
+    transform_mat = np.column_stack((u_vec, v_vec))  # shape (2,2)
+    try:
+        transform_inv = np.linalg.inv(transform_mat)
+    except np.linalg.LinAlgError:
+        # Fallback if u_vec/v_vec are degenerate (shouldn't happen for proper rectangles)
+        transform_inv = np.linalg.pinv(transform_mat)
+
+    # Iterate trees and accumulate ellipsoidal canopy surfaces
+    for _, row in tree_gdf.iterrows():
+        geom = row['geometry']
+        if geom is None or not hasattr(geom, 'x'):
+            continue
+
+        top_h = float(row.get('top_height', 0.0) or 0.0)
+        bot_h = float(row.get('bottom_height', 0.0) or 0.0)
+        dia = float(row.get('crown_diameter', 0.0) or 0.0)
+
+        # Sanity checks and clamps
+        if dia <= 0 or top_h <= 0:
+            continue
+        if bot_h < 0:
+            bot_h = 0.0
+        if bot_h > top_h:
+            top_h, bot_h = bot_h, top_h
+
+        R = dia / 2.0  # radius (meters)
+        a = max((top_h - bot_h) / 2.0, 0.0)
+        z0 = (top_h + bot_h) / 2.0
+        if a == 0:
+            # Flat disk between bot_h and top_h collapses; treat as constant top/bottom
+            a = 0.0
+
+        # Tree center in lon/lat
+        tree_lon = float(geom.x)
+        tree_lat = float(geom.y)
+
+        # Map tree center to (u,v) metric coordinates relative to origin
+        delta = np.array([tree_lon, tree_lat]) - origin
+        alpha_beta = transform_inv @ delta  # meters along u (alpha) and v (beta)
+        alpha_m = alpha_beta[0]
+        beta_m = alpha_beta[1]
+
+        # Determine affected index ranges (bounding box in grid indices)
+        # Convert radius in meters to index offsets along u and v
+        du_cells = int(R / adjusted_meshsize[0] + 2)
+        dv_cells = int(R / adjusted_meshsize[1] + 2)
+
+        i_center_idx = int(alpha_m / adjusted_meshsize[0])
+        j_center_idx = int(beta_m / adjusted_meshsize[1])
+
+        i_min = max(0, i_center_idx - du_cells)
+        i_max = min(nx - 1, i_center_idx + du_cells)
+        j_min = max(0, j_center_idx - dv_cells)
+        j_max = min(ny - 1, j_center_idx + dv_cells)
+
+        if i_min > i_max or j_min > j_max:
+            continue
+
+        # Slice cell center coords for local window
+        ic = i_centers_m[i_min:i_max + 1][:, None]  # shape (Ii,1)
+        jc = j_centers_m[j_min:j_max + 1][None, :]  # shape (1,Jj)
+
+        # Compute radial distance in meters in grid metric space
+        di = ic - alpha_m
+        dj = jc - beta_m
+        r = np.sqrt(di * di + dj * dj)
+
+        # Mask for points within crown radius
+        within = r <= R
+        if not np.any(within):
+            continue
+
+        # Ellipsoidal vertical profile
+        # Avoid numerical issues for r slightly > R due to precision
+        ratio = np.clip(r / max(R, 1e-9), 0.0, 1.0)
+        factor = np.sqrt(1.0 - ratio * ratio)
+        local_top = z0 + a * factor
+        local_bot = z0 - a * factor
+
+        # Apply mask; cells outside remain zero contribution
+        local_top_masked = np.where(within, local_top, 0.0)
+        local_bot_masked = np.where(within, local_bot, 0.0)
+
+        # Merge with maxima to represent union of crowns
+        canopy_top[i_min:i_max + 1, j_min:j_max + 1] = np.maximum(
+            canopy_top[i_min:i_max + 1, j_min:j_max + 1], local_top_masked
+        )
+        canopy_bottom[i_min:i_max + 1, j_min:j_max + 1] = np.maximum(
+            canopy_bottom[i_min:i_max + 1, j_min:j_max + 1], local_bot_masked
+        )
+
+    # Ensure bottom <= top everywhere
+    canopy_bottom = np.minimum(canopy_bottom, canopy_top)
+
+    return canopy_top, canopy_bottom
\ No newline at end of file
